@@ -15,7 +15,6 @@ from ldimbenchmark.benchmark_evaluation import evaluate_leakages
 from tabulate import tabulate
 from ldimbenchmark.benchmark_complexity import run_benchmark_complexity
 import matplotlib.pyplot as plt
-from multiprocessing import cpu_count
 import enlighten
 from ldimbenchmark.evaluation_metrics import (
     precision,
@@ -26,6 +25,7 @@ from ldimbenchmark.evaluation_metrics import (
     f1Score,
 )
 from concurrent.futures.process import ProcessPoolExecutor
+from sqlalchemy import create_engine
 
 
 def execute_experiment(experiment: MethodRunner):
@@ -163,13 +163,14 @@ def load_result(folder: str) -> Dict:
         evaluation_dataset_leakages, detected_leaks
     )
     evaluation_results["method"] = run_info["method"]
+    evaluation_results["method_version"] = run_info.get("method_version", None)
     evaluation_results["dataset"] = run_info["dataset"]
     evaluation_results["dataset_part"] = run_info.get("dataset_part", None)
     evaluation_results["dataset_id"] = run_info["dataset_id"]
     evaluation_results["dataset_derivations"] = run_info["dataset_options"]
     evaluation_results["hyperparameters"] = run_info["hyperparameters"]
     evaluation_results["matched_leaks_list"] = matched_list
-    evaluation_results["_folder"] = folder
+    evaluation_results["_folder"] = os.path.basename(os.path.dirname(folder))
     evaluation_results["executed_at"] = run_info.get("executed_at", None)
 
     return evaluation_results
@@ -334,7 +335,7 @@ class LDIMBenchmark:
         )
         param_matrix = pd.DataFrame(index=index).reset_index()
 
-        return [param_matrix.iloc[n].to_dict() for n in range(param_matrix.shape[0])]
+        return param_matrix.to_dict(orient="records")
 
     def add_local_methods(self, methods):
         """
@@ -404,6 +405,8 @@ class LDIMBenchmark:
         if len(self.methods_docker) > 0 and len(self.methods_local) > 0:
             raise ValueError("Cannot run local and docker methods at the same time")
 
+        logging.info("Starting Benchmark")
+        logging.info("Preparing Hyperparameters")
         hyperparameters_map = self._get_hyperparameters_for_methods_and_datasets(
             hyperparameters=self.hyperparameters,
             method_ids=[
@@ -413,17 +416,20 @@ class LDIMBenchmark:
             dataset_base_ids=[dataset.id for dataset in self.datasets],
         )
 
-        # Generate Experiments
+        # TODO: Move to parallel step execution step in run_benchmark, but still validate at least once
         for dataset in self.datasets:
             for method in self.methods_docker:
                 method_name = method.split(":")[0].split("/")[-1]
-                hyperparameter_list = [hyperparameters_map[method_name][dataset.id]]
+                self.hyperparameter_list = [
+                    hyperparameters_map[method_name][dataset.id]
+                ]
                 if self.multi_parameters:
-                    hyperparameter_list = LDIMBenchmark._get_hyperparameters_matrix_from_hyperparameters_with_list(
+                    self.hyperparameter_list = LDIMBenchmark._get_hyperparameters_matrix_from_hyperparameters_with_list(
                         hyperparameters_map[method_name][dataset.id]
                     )
 
-                for hyperparameters in hyperparameter_list:
+                logging.info(f"Generating {len(self.hyperparameter_list)} Experiments")
+                for hyperparameters in self.hyperparameter_list:
                     self.experiments.append(
                         DockerMethodRunner(
                             method,
@@ -436,13 +442,16 @@ class LDIMBenchmark:
                     )
 
             for method in self.methods_local:
-                hyperparameter_list = [hyperparameters_map[method.name][dataset.id]]
+                self.hyperparameter_list = [
+                    hyperparameters_map[method.name][dataset.id]
+                ]
                 if self.multi_parameters:
-                    hyperparameter_list = LDIMBenchmark._get_hyperparameters_matrix_from_hyperparameters_with_list(
+                    self.hyperparameter_list = LDIMBenchmark._get_hyperparameters_matrix_from_hyperparameters_with_list(
                         hyperparameters_map[method.name][dataset.id]
                     )
 
-                for hyperparameters in hyperparameter_list:
+                logging.info(f"Generating {len(self.hyperparameter_list)} Experiments")
+                for hyperparameters in self.hyperparameter_list:
                     self.experiments.append(
                         LocalMethodRunner(
                             detection_method=method,
@@ -477,8 +486,15 @@ class LDIMBenchmark:
                 justify=enlighten.Justify.CENTER,
             )
         results = []
+        bar_experiments = manager.counter(
+            total=num_experiments,
+            desc="Experiments",
+            unit="experiments",
+            count=num_experiments - len(self.experiments),
+        )
+        # This line makes sure we can call update with an effect
         if parallel:
-            worker_num = cpu_count() - 1
+            worker_num = os.cpu_count() - 1
             if parallel_max_workers > 0:
                 worker_num = parallel_max_workers
             try:
@@ -488,31 +504,27 @@ class LDIMBenchmark:
                         executor.submit(execute_experiment, runner)
                         for runner in self.experiments
                     ]
-                    pbar = manager.counter(
-                        total=num_experiments, desc="Experiments", unit="experiments"
-                    )
-                    print(num_experiments - len(self.experiments))
-                    pbar.update(incr=num_experiments - len(self.experiments))
-
                     # process results from tasks in order of task completion
                     for future in as_completed(futures):
                         future.result()
-                        pbar.update()
-                    pbar.close()
+                        logging.info("update bar")
+                        bar_experiments.update()
             except KeyboardInterrupt:
-                manager.stop()
+                executor.shutdown(wait=False)
                 executor._processes.clear()
                 os.kill(os.getpid(), 9)
+                manager.stop()
         else:
             for experiment in self.experiments:
                 results.append(experiment.run())
+                bar_experiments.update()
         manager.stop()
 
     def evaluate(
         self,
         current_only=True,
         resultFilter: Callable = lambda r: r,
-        write_results=False,
+        write_results: Union[None, Literal["csv", "db"]] = None,
         generate_plots=False,
         evaluations: List[Callable] = [
             precision,
@@ -560,11 +572,16 @@ class LDIMBenchmark:
                 f"You are generating Plots for {len(result_folders)} results! This will take ages, consider only generating them for the benchmark run you are interested in."
             )
 
-        logging.info(f"Loading {len(result_folders)} results...")
+        manager = enlighten.get_manager()
+        pbar1 = manager.counter(
+            total=len(result_folders),
+            desc="Loading Results",
+            unit="results",
+        )
         results = []
         parallel = True
         if parallel == True:
-            with ProcessPoolExecutor(max_workers=cpu_count() - 1) as executor:
+            with ProcessPoolExecutor() as executor:
                 # submit all tasks and get future objects
                 futures = [
                     executor.submit(load_result, folder) for folder in result_folders
@@ -573,9 +590,11 @@ class LDIMBenchmark:
                 for future in as_completed(futures):
                     result = future.result()
                     results.append(result)
+                    pbar1.update()
         else:
             for experiment_result in result_folders:
                 results.append(load_result(experiment_result))
+        pbar1.close()
 
         if generate_plots:
             logging.info("Generating plots...")
@@ -587,7 +606,6 @@ class LDIMBenchmark:
                     loaded = dataset
 
                 loaded_datasets[dataset.id] = loaded.loadData()
-            manager = enlighten.get_manager()
             pbar = manager.counter(total=len(results), desc="Results:", unit="results")
 
             for result in results:
@@ -601,7 +619,7 @@ class LDIMBenchmark:
                 )
                 parallel = True
                 if parallel:
-                    with ProcessPoolExecutor(max_workers=cpu_count() - 1) as executor:
+                    with ProcessPoolExecutor() as executor:
                         # submit all tasks and get future objects
                         futures = []
                         for leak_pair in result["matched_leaks_list"]:
@@ -633,7 +651,8 @@ class LDIMBenchmark:
                         pbar2.update()
                 pbar2.close()
                 pbar.update()
-            manager.close()
+
+        manager.stop()
         results = pd.DataFrame(results)
 
         for function in evaluations:
@@ -641,11 +660,42 @@ class LDIMBenchmark:
 
         results = resultFilter(results)
         # https://towardsdatascience.com/performance-metrics-confusion-matrix-precision-recall-and-f1-score-a8fe076a2262
-        results = results.set_index(["method", "dataset_id"])
-        results = results.sort_values(by=["F1"])
+        results = results.set_index(["_folder"])
+        results = results.drop(columns=["matched_leaks_list"])
 
         os.makedirs(self.evaluation_results_dir, exist_ok=True)
-        results = results.drop(columns=["_folder", "matched_leaks_list"])
+
+        if write_results == "csv":
+            print("Writing results to disk")
+            results.to_csv(os.path.join(self.evaluation_results_dir, "results.csv"))
+
+            results.style.format(escape="latex").set_table_styles(
+                [
+                    # {'selector': 'toprule', 'props': ':hline;'},
+                    {"selector": "midrule", "props": ":hline;"},
+                    # {'selector': 'bottomrule', 'props': ':hline;'},
+                ],
+                overwrite=False,
+            ).relabel_index(columns, axis="columns").to_latex(
+                os.path.join(self.evaluation_results_dir, "results.tex"),
+                position_float="centering",
+                clines="all;data",
+                column_format="ll|" + "r" * len(columns),
+                position="H",
+                label="table:benchmark_results",
+                caption="Overview of the benchmark results.",
+            )
+        elif write_results == "db":
+            print("Writing results to database")
+            engine = create_engine(
+                f"sqlite:///{os.path.join(self.evaluation_results_dir, 'results.db')}"
+            )
+            results.to_sql("results", engine, if_exists="replace")
+
+        results = results.set_index(["method", "method_version", "dataset_id"])
+        results = results.sort_values(by=["F1"])
+        # Display in console
+        # results = results.drop(columns=["_folder", "matched_leaks_list"])
         # TODO: Automatically add selected metrics
         columns = [
             "TP",
@@ -670,27 +720,6 @@ class LDIMBenchmark:
         results.columns = columns
 
         print(tabulate(results, headers="keys"))
-
-        if write_results:
-            print("Writing results to disk")
-            results.to_csv(os.path.join(self.evaluation_results_dir, "results.csv"))
-
-            results.style.format(escape="latex").set_table_styles(
-                [
-                    # {'selector': 'toprule', 'props': ':hline;'},
-                    {"selector": "midrule", "props": ":hline;"},
-                    # {'selector': 'bottomrule', 'props': ':hline;'},
-                ],
-                overwrite=False,
-            ).relabel_index(columns, axis="columns").to_latex(
-                os.path.join(self.evaluation_results_dir, "results.tex"),
-                position_float="centering",
-                clines="all;data",
-                column_format="ll|" + "r" * len(columns),
-                position="H",
-                label="table:benchmark_results",
-                caption="Overview of the benchmark results.",
-            )
         return results
 
 
