@@ -1,4 +1,4 @@
-from multiprocessing import Pool, cpu_count
+from concurrent.futures import ThreadPoolExecutor, as_completed
 import pickle
 import numpy as np
 import pandas as pd
@@ -125,7 +125,8 @@ def loadDatasetsDirectly(
                 index_col="Timestamp",
                 chunksize=100000,
             )
-            frames = Pool(processes=cpu_count() - 1).imap(parse_frame_dates, d)
+            with ThreadPoolExecutor(max_workers=os.cpu_count() - 1) as executor:
+                frames = list(executor.map(parse_frame_dates, d))
 
             sensor_readings = pd.concat(frames)
 
@@ -350,10 +351,17 @@ class Dataset:
                 for sensor in getattr(self, sensor_type).keys()
             ]
             # logging.debug(filepaths)
-            with Pool(processes=cpu_count() - 1) as p:
-                p.starmap(
-                    write_to_csv, zip(getattr(self, sensor_type).values(), filepaths)
-                )
+            with ThreadPoolExecutor(max_workers=os.cpu_count() - 1) as executor:
+                # submit all tasks and get future objects
+                futures = [
+                    executor.submit(write_to_csv, sensor, path)
+                    for sensor, path in zip(
+                        getattr(self, sensor_type).values(), filepaths
+                    )
+                ]
+                # process results from tasks in order of task completion
+                for future in as_completed(futures):
+                    future.result()
 
         self.leaks.to_csv(os.path.join(folder, "leaks.csv"))
 
@@ -364,7 +372,11 @@ class Dataset:
 
 
 def getTimeSliceOfDataset(
-    dataset: Dict[str, DataFrame], start: datetime, end: datetime
+    dataset: Dict[str, DataFrame],
+    start: datetime,
+    end: datetime,
+    logging_dataset_name: str,
+    logging_sensor_type: str,
 ) -> DataFrame:
     """
     Get a time slice of a dataframe.
@@ -375,9 +387,10 @@ def getTimeSliceOfDataset(
         logging.debug(key)
         new_dataset_slice[key] = dataset[key].sort_index().loc[start:end]
 
-    if len(new_dataset_slice) == 0:
+    # If there is no data in the slice, but there is data in the dataset, then the start- and endtime are outside of the datapoint ranges.
+    if len(new_dataset_slice) == 0 and len(dataset.keys()) != 0:
         logging.warning(
-            f"No data from dataset selected, because start- and endtime ({start} / {end}) are outside of the datapoint ranges."
+            f"No data from dataset '{logging_dataset_name}' with sensor type '{logging_sensor_type}' selected, because start- and endtime ({start} / {end}) are outside of the datapoint ranges."
         )
     return new_dataset_slice
 
@@ -401,11 +414,17 @@ def extractSubDataset(
     return _LoadedDatasetPartNew(
         {
             "pressures": getTimeSliceOfDataset(
-                full_data.pressures, start_time, end_time
+                full_data.pressures, start_time, end_time, config["name"], "pressures"
+            ),
+            "demands": getTimeSliceOfDataset(
+                full_data.demands, start_time, end_time, config["name"], "demands"
+            ),
+            "flows": getTimeSliceOfDataset(
+                full_data.flows, start_time, end_time, config["name"], "flows"
+            ),
+            "levels": getTimeSliceOfDataset(
+                full_data.levels, start_time, end_time, config["name"], "levels"
             ),
-            "demands": getTimeSliceOfDataset(full_data.demands, start_time, end_time),
-            "flows": getTimeSliceOfDataset(full_data.flows, start_time, end_time),
-            "levels": getTimeSliceOfDataset(full_data.levels, start_time, end_time),
             "leaks": leaks,
         }
     )
