@@ -52,6 +52,12 @@ class LILA(LDIMMethodBase):
                         description="End date of a leakfree period of time inside the dataset. Format should be 'YYYY-MM-DD'. Leaving it empty will use the whole dataset.",
                         value_type=str,
                     ),
+                    Hyperparameter(
+                        name="resample_frequency",
+                        description="Time frequency for resampling the data. e.g. '1T' for 1 minute, '1H' for 1 hour, '1D' for 1 day.",
+                        value_type=str,
+                        default="5T",
+                    ),
                     Hyperparameter(
                         name="est_length",
                         description="Length of the estimation period in hours",
@@ -113,6 +119,7 @@ class LILA(LDIMMethodBase):
                 "J-02": "PUMP_1",
                 "Inflow [l/s]": "PUMP_1",
                 "31664": "PUMP_1",
+                "wNode_1": "PUMP_1",
             }
         )
         scada_data.flows = flows
@@ -159,7 +166,8 @@ class LILA(LDIMMethodBase):
     def prepare(self, training_data: BenchmarkData = None) -> None:
         if training_data != None:
             simple_training_data = simplifyBenchmarkData(
-                training_data, resample_frequency="5T"
+                training_data,
+                resample_frequency=self.hyperparameters["resample_frequency"],
             )
 
             start_time = pd.to_datetime(self.hyperparameters["leakfree_time_start"])
@@ -172,11 +180,12 @@ class LILA(LDIMMethodBase):
         scada_data = SCADA_data()
 
         simple_evaluation_data = simplifyBenchmarkData(
-            evaluation_data, resample_frequency="5T"
+            evaluation_data,
+            resample_frequency=self.hyperparameters["resample_frequency"],
         )
 
         if self.trained == False:
-            # TODO: Implement reoccuring training on trailing timeframe?
+            # TODO: Implement reoccurring training on trailing timeframe?
             start_time = pd.to_datetime(self.hyperparameters["leakfree_time_start"])
             end_time = pd.to_datetime(self.hyperparameters["leakfree_time_stop"])
             self.train(simple_evaluation_data, start_time, end_time)
@@ -188,7 +197,12 @@ class LILA(LDIMMethodBase):
         # TODO: Make algorithm independent of pump name
         # flows["PUMP_1"] = flows.sum(axis=1)
         flows = flows.rename(
-            columns={"P-01": "PUMP_1", "J-02": "PUMP_1", "Inflow [l/s]": "PUMP_1"}
+            columns={
+                "P-01": "PUMP_1",
+                "J-02": "PUMP_1",
+                "Inflow [l/s]": "PUMP_1",
+                "wNode_1": "PUMP_1",
+            }
         )
 
         scada_data.flows = flows
